@@ -1,11 +1,16 @@
-use std::sync::mpsc::Sender;
+use std::sync::{
+    atomic::{AtomicBool, AtomicU16, Ordering},
+    mpsc::{Sender, SyncSender},
+    Arc,
+};
 
-use crate::{utils::FileContent, view::AlignedMessage};
+use crate::{file::FileContent, view::AlignedMessage};
 use bio::alignment::{
     pairwise::{self, MatchFunc, Scoring},
-    Alignment, AlignmentOperation as Op,
+    AlignmentOperation as Op,
 };
-use serde::{Serialize, Deserialize};
+use realfft::{num_complex::Complex32, RealFftPlanner, RealToComplex};
+use serde::{Deserialize, Serialize};
 
 pub const DEFAULT_BLOCKSIZE: usize = 8192;
 pub const DEFAULT_KMER: usize = 8;
@@ -31,7 +36,10 @@ pub enum Banded {
     Banded { kmer: usize, window: usize },
 }
 impl Banded {
-    fn align<F: MatchFunc>(&self, scoring: Scoring<F>, x: &[u8], y: &[u8]) -> Alignment {
+    fn align<F: MatchFunc>(&self, scoring: Scoring<F>, x: &[u8], y: &[u8]) -> Vec<Op> {
+        if x == y {
+            return vec![Op::Match; x.len()];
+        }
         // note that we recreate the Aligner each call, but i don't think that part is expensive
         match self {
             Banded::Normal => pairwise::Aligner::with_scoring(scoring).custom(x, y),
@@ -39,11 +47,13 @@ impl Banded {
                 pairwise::banded::Aligner::with_scoring(scoring, *kmer, *window).custom(x, y)
             }
         }
+        .operations
     }
 }
 
 /// Contains parameters to run the alignment algorithm with
 #[derive(Clone, Debug, Serialize, Deserialize)]
+#[serde(default)]
 pub struct AlignAlgorithm {
     pub gap_open: i32,
     pub gap_extend: i32,
@@ -198,7 +208,7 @@ fn align_whole<F: MatchFunc>(
     sender: Sender<AlignedMessage>,
 ) {
     let _ = sender.send(AlignedMessage::Append(
-        AlignElement::from_array(&band.align(scoring, &x, &y).operations, &x, &y, 0, 0).0,
+        AlignElement::from_array(&band.align(scoring, &x, &y), &x, &y, 0, 0).0,
     ));
 }
 
@@ -227,7 +237,7 @@ pub fn align_end<F: MatchFunc + Clone>(
         );
         // we only actually append at most half of the block size since we make sure gaps crossing
         // block boundaries are better detected
-        let ops = &end_aligned.operations[0..end_aligned.operations.len().min(block_size / 2)];
+        let ops = &end_aligned[0..end_aligned.len().min(block_size / 2)];
         // we will not progress like this, so might as well quit
         // might indicate an error
         if ops.is_empty() {
@@ -267,8 +277,7 @@ pub fn align_front<F: MatchFunc + Clone>(
         // unlike in align_end, we create the Alignelement from the whole array and then cut it
         // in half. This is because the addresses returned from from_array are at the end, which
         // we already know, so we instead take the start addresses from the array itself
-        let (end, _, _) =
-            AlignElement::from_array(&aligned.operations, &x, &y, lower_xaddr, lower_yaddr);
+        let (end, _, _) = AlignElement::from_array(&aligned, &x, &y, lower_xaddr, lower_yaddr);
         let real_end = Vec::from(&end[end.len().saturating_sub(block_size / 2)..end.len()]);
         // if this is empty, we will not progress, so just quit
         if real_end.is_empty() {
@@ -282,3 +291,145 @@ pub fn align_front<F: MatchFunc + Clone>(
         }
     }
 }
+
+pub enum FlatAlignProgressMessage {
+    Incomplete(u16),
+    Complete(isize),
+}
+pub struct FlatAlignmentContext {
+    is_running: Arc<AtomicBool>,
+    vecs: [FileContent; 2],
+    update_progress: Box<dyn FnMut(FlatAlignProgressMessage) + Send + 'static>,
+}
+
+impl FlatAlignmentContext {
+    pub fn new(
+        is_running: Arc<AtomicBool>,
+        vecs: [FileContent; 2],
+        update_progress: Box<dyn FnMut(FlatAlignProgressMessage) + Send + 'static>,
+    ) -> Self {
+        Self {
+            is_running,
+            vecs,
+            update_progress,
+        }
+    }
+
+    // this finds the alignment between two arrays *without* removing elements such that
+    // fewest bytes are different (for the compvec)
+    pub fn align_flat(mut self) {
+        // this algorithm works by, for each byte:
+        // * making an indicator vector for both files indicating the addresses that have the given byte
+        // * cross-correlating them, which results in the number of matches of that byte value for each relative offset
+        // and then adding them all together to get the total number of matching bytes
+        let mut progress = 0u16;
+        let current_byte = Arc::new(AtomicU16::new(0));
+        let mut fft_planner = RealFftPlanner::new();
+        let total_len = self.vecs.iter().map(|x| x.len()).max().unwrap() * 2;
+        // the cross correlation is done using the omnipresent fft algorithm
+        let fft_forward = fft_planner.plan_fft_forward(total_len);
+        let fft_inverse = fft_planner.plan_fft_inverse(total_len);
+        let mut sum = fft_forward.make_output_vec();
+        // this is easily parallelizable for up to 256 threads, for which we span a thread pool
+        let thread_num = num_cpus::get().clamp(1, 256);
+        let (send, recv) = std::sync::mpsc::sync_channel::<Vec<Complex32>>(4.max(thread_num));
+        for _ in 0..thread_num {
+            let vecs = [self.vecs[0].clone(), self.vecs[1].clone()];
+            let inbyte = current_byte.clone();
+            let outvecs = send.clone();
+            let fft = fft_forward.clone();
+            std::thread::spawn(move || correlation_thread(vecs, inbyte, outvecs, fft));
+        }
+        for vec in recv.into_iter().take(256) {
+            if !self.is_running.load(Ordering::Relaxed) {
+                return;
+            }
+            // add the vectors together in the frequency domain
+            for (a, b) in sum.iter_mut().zip(vec.into_iter()) {
+                *a += b;
+            }
+            progress += 1;
+            (self.update_progress)(FlatAlignProgressMessage::Incomplete(progress));
+        }
+        // get the actual result in the time domain
+        let mut result = fft_inverse.make_output_vec();
+        fft_inverse
+            .process(&mut sum, &mut result)
+            .expect("Wrong lengths");
+        drop(sum);
+        // positive offset of the array with the highest value of overlap
+        let offset = result
+            .iter()
+            .enumerate()
+            .max_by(|a, b| {
+                a.1.partial_cmp(b.1).unwrap_or_else(|| {
+                    if a.1.is_nan() {
+                        std::cmp::Ordering::Less
+                    } else {
+                        std::cmp::Ordering::Greater
+                    }
+                })
+            })
+            .unwrap_or((0, &0.0))
+            .0;
+        drop(result);
+        // reverse direction of result array
+        let offset = total_len - offset - 1;
+        // get the relative offset between the two vectors with optimal overlap
+        let relative_offset = if offset >= total_len / 2 {
+            offset as isize - total_len as isize
+        } else {
+            offset as isize
+        };
+        (self.update_progress)(FlatAlignProgressMessage::Complete(relative_offset))
+    }
+}
+
+fn correlation_thread(
+    vecs: [FileContent; 2],
+    inbyte: Arc<AtomicU16>,
+    outvecs: SyncSender<Vec<Complex32>>,
+    fft: Arc<dyn RealToComplex<f32>>,
+) {
+    let len = fft.len();
+    loop {
+        // check if the next value in queue is still below 256
+        let byte: u8 = match inbyte.fetch_add(1, Ordering::Relaxed).try_into() {
+            Ok(f) => f,
+            Err(_) => return,
+        };
+
+        // cross-correlation using ffts
+        let mut first_out = fft.make_output_vec();
+        let mut first = fft.make_input_vec();
+        // one of the vectors is reversed because we want correlation, not convolution
+        for (i, x) in vecs[0].iter().enumerate() {
+            if *x == byte {
+                first[len - i - 1] = 1.0;
+            }
+        }
+        fft.process(&mut first, &mut first_out)
+            .expect("Wrong fft vector lengths");
+        // these vectors can be large, so drop them as soon as possible
+        drop(first);
+        let mut second = fft.make_input_vec();
+        for (i, x) in vecs[1].iter().enumerate() {
+            if *x == byte {
+                second[i] = 1.0
+            }
+        }
+        let mut second_out = fft.make_output_vec();
+        fft.process(&mut second, &mut second_out)
+            .expect("Wrong fft vector lengths");
+        drop(second);
+        for (a, b) in first_out.iter_mut().zip(second_out.iter()) {
+            *a *= b;
+        }
+        drop(second_out);
+        // note: we do not correlate fully, since we can add all the samples together
+        // in the frequency domain, saving nearly 1/3 of the processing time
+        if outvecs.send(first_out).is_err() {
+            return;
+        }
+    }
+}
