@@ -36,6 +36,12 @@ pub enum Action {
     Algorithm,
     Refresh,
     Goto,
+    Search,
+    SetOffset,
+    NextSearch,
+    PrevSearch,
+    Top,
+    Bottom,
     CursorFirst,
     CursorBoth,
     CursorSecond,
@@ -70,6 +76,9 @@ impl TryFrom<Event> for Action {
                 KeyCode::Char('j') => Action::DownAlt,
                 KeyCode::Char('k') => Action::UpAlt,
                 KeyCode::Char('l') => Action::RightAlt,
+                KeyCode::Char('n') => Action::NextSearch,
+                KeyCode::Char('N') => Action::PrevSearch,
+                KeyCode::Char('o') => Action::SetOffset,
                 KeyCode::Char(' ') => Action::NextDifference,
                 KeyCode::F(1) => Action::Help,
                 KeyCode::Char('1') => Action::Help,
@@ -83,6 +92,10 @@ impl TryFrom<Event> for Action {
                 KeyCode::Char('5') => Action::Refresh,
                 KeyCode::F(6) => Action::Goto,
                 KeyCode::Char('6') => Action::Goto,
+                KeyCode::F(7) => Action::Search,
+                KeyCode::Char('7') => Action::Search,
+                KeyCode::Home => Action::Top,
+                KeyCode::End => Action::Bottom,
                 _ => return Err(()),
             },
         )
@@ -176,6 +189,7 @@ impl Color {
 #[derive(Clone, Copy, Debug)]
 pub enum Effect {
     Inverted,
+    Bold,
     None,
 }
 
@@ -185,14 +199,16 @@ impl Effect {
             // this assumes that no other effects are in effect
             // so change this when adding others
             // note that Reset also resets colors
-            Effect::None => Attribute::NoReverse,
+            Effect::None => Attribute::Reset,
             Effect::Inverted => Attribute::Reverse,
+            Effect::Bold => Attribute::Bold,
         }
     }
     fn to_cursiv(self) -> theme::Effect {
         match self {
             Effect::None => theme::Effect::Simple,
             Effect::Inverted => theme::Effect::Reverse,
+            Effect::Bold => theme::Effect::Bold,
         }
     }
 }
@@ -289,15 +305,14 @@ impl Backend for Cross {
         if Some(attribute) != self.prev_effect {
             queue!(
                 self.buffer,
+                style::SetAttribute(Attribute::Reset),
                 style::SetAttribute(attribute),
                 style::SetBackgroundColor(CrossColor::Black)
             )
             .unwrap_or_else(quit_with_error("Could not write out text"));
             self.prev_effect = Some(attribute);
-            // if the attribute is Reset, then we also need to set the color again
-            if matches!(attribute, Attribute::Reset) {
-                self.prev_color = None;
-            }
+            // because the attribute is Reset, then we also need to set the color again
+            self.prev_color = None;
         }
         let cross_color = color.to_cross();
         if Some(cross_color) != self.prev_color {
@@ -360,10 +375,12 @@ impl Backend for Cross {
 
     fn clear(&mut self) {
         self.prev_effect = Some(Attribute::NoReverse);
-        queue!(self.buffer, 
+        queue!(
+            self.buffer,
             style::SetAttribute(Attribute::NoReverse),
-            terminal::Clear(terminal::ClearType::All),)
-            .unwrap_or_else(quit_with_error("Could not clear screen"))
+            terminal::Clear(terminal::ClearType::All),
+        )
+        .unwrap_or_else(quit_with_error("Could not clear screen"))
     }
 }
 
@@ -421,7 +438,7 @@ impl<'a, 'b, 'c> Backend for Cursiv<'a, 'b, 'c> {
     }
 }
 
-pub struct Dummy {}
+pub struct Dummy;
 
 impl Backend for Dummy {
     fn set_line(&mut self, _: usize) {}
