@@ -1,6 +1,6 @@
 use std::ops::Range;
 
-use crate::utils::FileContent;
+use crate::{file::FileContent, util::entropy};
 
 /// trait for arrays with signed index
 pub trait SignedArray {
@@ -52,6 +52,14 @@ impl<T: Clone> DoubleVec<T> {
     pub fn extend_end(&mut self, other: &[T]) {
         self.end.extend_from_slice(other);
     }
+
+    pub fn first(&self) -> Option<&T> {
+        self.front.last().or_else(|| self.end.first())
+    }
+
+    pub fn last(&self) -> Option<&T> {
+        self.end.last().or_else(|| self.front.first())
+    }
 }
 
 impl<T: Clone> SignedArray for DoubleVec<T> {
@@ -71,7 +79,8 @@ impl<T: Clone> SignedArray for DoubleVec<T> {
 }
 
 /// A vector consisting of two arrays, the second one being at an
-/// offset of the second one.
+/// offset of the second one, with at least one byte of overlap
+#[derive(Clone)]
 pub struct CompVec {
     pub xvec: FileContent,
     pub yvec: FileContent,
@@ -87,15 +96,28 @@ impl CompVec {
             shift: 0,
         }
     }
+    // adds relative_shift to self.shift, ensuring that the arrays
+    // still overlap and returns the actual relative shift applied
+    fn modify_shift(&mut self, relative_shift: isize) -> isize {
+        // they obviously cannot overlap if they're both empty, so just do nothing
+        if self.yvec.is_empty() && self.xvec.is_empty() {
+            return 0;
+        }
+        let shift_range = -(self.yvec.len() as isize - 1)..self.xvec.len() as isize;
+        let old_shift = self.shift;
+        let proposed_shift = old_shift + relative_shift;
+        self.shift = proposed_shift.clamp(shift_range.start, shift_range.end - 1);
+
+        self.shift - old_shift
+    }
     /// Adds a shift to the left vector, returns the change of index to the same element
     pub fn add_first_shift(&mut self, relative_shift: isize) -> isize {
-        self.shift -= relative_shift;
+        self.modify_shift(-relative_shift);
         -relative_shift
     }
     /// Adds a shift to the right vector, returns the change of index to the same element
     pub fn add_second_shift(&mut self, relative_shift: isize) -> isize {
-        self.shift += relative_shift;
-        0
+        -relative_shift + self.modify_shift(relative_shift)
     }
     /// Gets the address of the left vector at a given index
     pub fn get_first_addr(&self, index: isize) -> Option<usize> {
@@ -118,6 +140,85 @@ impl CompVec {
     pub fn get_data(&self) -> (FileContent, FileContent) {
         (self.xvec.clone(), self.yvec.clone())
     }
+    /// Returns the possible indexes where the first vector has data
+    pub fn first_bound(&self) -> Range<isize> {
+        0..self.xvec.len() as isize
+    }
+    /// Returns the possible indexes where the second vector has data
+    pub fn second_bound(&self) -> Range<isize> {
+        self.shift..self.shift as isize + self.yvec.len() as isize
+    }
+    /// Calculates the index of a subsequence (with the current offset)
+    /// where the product of entropy and length is the highest
+    pub fn highest_common_entropy(&self) -> isize {
+        if self.xvec.is_empty() || self.yvec.is_empty() {
+            return 0;
+        }
+        let first_start_index = 0.max(self.shift) as usize;
+        let second_start_index = 0.max(-self.shift) as usize;
+        let max = CommonSequenceIterator::new(
+            &self.xvec[first_start_index..],
+            &self.yvec[second_start_index..],
+        )
+        .map(|(data, range)| {
+            let entr = entropy(data);
+            (entr * range.len() as f32, range.start)
+        })
+        .max_by(|a, b| {
+            // we prefer the one that is not nan for the maximum
+            a.0.partial_cmp(&b.0).unwrap_or_else(|| {
+                if a.0.is_nan() {
+                    std::cmp::Ordering::Less
+                } else {
+                    std::cmp::Ordering::Greater
+                }
+            })
+        });
+        let (_, addr) = match max {
+            Some(x) => x,
+            None => return 0,
+        };
+        (first_start_index + addr) as isize
+    }
+}
+
+/// Iterates over common sequences of the two given vectors
+pub struct CommonSequenceIterator<'a> {
+    idx: usize,
+    a: &'a [u8],
+    b: &'a [u8],
+}
+
+impl<'a> CommonSequenceIterator<'a> {
+    /// Returns the CommonSequenceIterator over the given vectors
+    pub fn new(a: &'a [u8], b: &'a [u8]) -> Self {
+        Self { idx: 0, a, b }
+    }
+}
+
+impl<'a> Iterator for CommonSequenceIterator<'a> {
+    type Item = (&'a [u8], Range<usize>);
+
+    fn next(&mut self) -> Option<Self::Item> {
+        let next_common_index = self
+            .a
+            .iter()
+            .zip(self.b.iter())
+            .enumerate()
+            .find_map(|(i, (a, b))| (a == b).then(|| i))?;
+        let next_uncommon_index = self.a[next_common_index..]
+            .iter()
+            .zip(self.b[next_common_index..].iter())
+            .enumerate()
+            .find_map(|(i, (a, b))| (a != b).then(|| i + next_common_index))
+            .unwrap_or_else(|| self.a.len().min(self.b.len()));
+        let ret = &self.a[next_common_index..next_uncommon_index];
+        self.a = &self.a[next_uncommon_index..];
+        self.b = &self.b[next_uncommon_index..];
+        let range = self.idx + next_common_index..self.idx + next_uncommon_index;
+        self.idx += next_uncommon_index;
+        Some((ret, range))
+    }
 }
 
 impl SignedArray for CompVec {
@@ -212,4 +313,21 @@ mod tests {
         assert_eq!(v.binary_search(&Some(6), Option::<i32>::cmp), Err(0));
         assert_eq!(v.binary_search(&Some(0), Option::<i32>::cmp), Err(-3));
     }
+    #[test]
+    fn common_sequence_iter() {
+        let common_sequences = CommonSequenceIterator::new(
+            &[0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 0, 12, 13, 14, 15, 16],
+            &[0, 1, 0, 3, 4, 5, 0, 7, 8, 9, 0, 11, 12, 13, 14, 15, 16, 17],
+        )
+        .collect::<Vec<_>>();
+        assert_eq!(
+            common_sequences,
+            vec![
+                (&[0, 1][..], 0..2),
+                (&[3, 4, 5][..], 3..6),
+                (&[7, 8, 9][..], 7..10),
+                (&[12, 13, 14, 15, 16][..], 12..17)
+            ]
+        )
+    }
 }
